#==========================================================================
#              Copyright (c) 1995-1998 Martien Verbruggen
#              Copyright (c) 2001-  Kawai Takanori
#--------------------------------------------------------------------------
#
#   Name:
#       GD::Graph::candle.pm
#
# $Id: candle.pm,v 0.01 2001/01/25 20:00:00 mgjv Exp $
#
#==========================================================================
 
package GD::Graph::candle;

$GD::Graph::candle::VERSION = '$Revision: 0.01 $' =~ /\s([\d.]+)/;

use strict;

use GD;
use GD::Graph::axestype;

@GD::Graph::candle::ISA = qw(GD::Graph::axestype);

sub set
{
    my($self, %args) =@_;
    $self->SUPER::set(%args);
#1. Set candle style (default:Japanese Style)
    $self->{candle_style} = $args{candle_style} 
            if(defined $args{candle_style} );
}
sub check_data
{
    my($self, $data) =@_;
#1. Check with SUPER class
    $self->SUPER::check_data($data) or return;

#2. Check only for candle
    my $i=0;
    my $iCnt;
    foreach my $rVal (@$data) {
        next if($i++ <= 0); #Skip Label
        foreach my $rItem (@$rVal) {
            $iCnt = scalar @$rItem unless($iCnt);
            #Must be 3 or 4
            return $self->_set_error('Data sets must be 3 or 4')
                    if(($iCnt != 3) and ($iCnt != 4));
            #Must be the same
            return $self->_set_error('Unmatch count')
                    if($iCnt != (scalar @$rItem)) ;
        }
    }
    return $self;
}

sub draw_data_set
{
    my($self, $ds) = @_;

#1. Get data from data set
    my @values = $self->{_data}->y_values($ds) or
        return $self->_set_error("Impossible illegal data set: $ds",
            $self->{_data}->error);

#2. Calc bar-space and get color
    my $bar_s = $self->{bar_spacing}/2;
    # Pick a data colour
    my $dsci = $self->set_clr($self->pick_data_clr($ds));

#3. Draw chart
    for (my $i = 0; $i < @values; $i++) {
        my $value = $values[$i];
        next unless defined $value;

        #3.1 calculate left and right of bar
        my ($xp, $dum) = $self->val_to_pixel($i+1, $value->[0], $ds);
        my ($l, $r, $m);
        if (ref $self eq 'GD::Graph::mixed' || $self->{overwrite})
        {
            $l = $xp - $self->{x_step}/2 + $bar_s + 1;
            $r = $xp + $self->{x_step}/2 - $bar_s;
        }
        else
        {
            $l = $xp 
                - $self->{x_step}/2
                + ($ds - 1) * $self->{x_step}/$self->{_data}->num_sets
                + $bar_s + 1;
            $r = $xp 
                - $self->{x_step}/2
                + $ds * $self->{x_step}/$self->{_data}->num_sets
                - $bar_s;
        }
        $m = int(($l+$r)/2);

        my ($iLowX, $iLowY, $iHighX, $iHighY, $iCloseX, $iCloseY);
        # Draw Barchart (High-Low-Close)
        if(scalar(@$value) <= 3) {
            ($iHighX, $iHighY)  = $self->val_to_pixel($i+1, $value->[0], $ds);
            ($iLowX,  $iLowY)   = $self->val_to_pixel($i+1, $value->[1], $ds);
            ($iCloseX,$iCloseY) = $self->val_to_pixel($i+1, $value->[2], $ds);
            $self->{graph}->line( $m, $iLowY, $m, $iHighY, $dsci);
            $self->{graph}->line( $m, $iCloseY, $r, $iCloseY, $dsci);
        }
        else {
        # Draw Candlestick (Open-High-Low-Close)
            my ($iOpenX, $iOpenY);
            ($iOpenX, $iOpenY)  = $self->val_to_pixel($i+1, $value->[0], $ds);
            ($iHighX, $iHighY)  = $self->val_to_pixel($i+1, $value->[1], $ds);
            ($iLowX,  $iLowY)   = $self->val_to_pixel($i+1, $value->[2], $ds);
            ($iCloseX,$iCloseY) = $self->val_to_pixel($i+1, $value->[3], $ds);
        #Default Japanese Candle
            if(!defined($self->{candle_style})) { 
                #Body
                if($iCloseY < $iOpenY) {  # Open price < Close Price
                    my $iWhite = $self->{graph}->colorResolve(255,255,255);
                    $self->{graph}->filledRectangle($l, $iCloseY, $r, $iOpenY, $iWhite);
                    $self->{graph}->rectangle($l, $iOpenY, $r, $iCloseY, $dsci);
                }
                else {  # Open price > Close Price
                    $self->{graph}->filledRectangle($l, $iOpenY, $r, $iCloseY, $dsci);
                }
                #Foot
                my $iIeMax = ($iOpenY<$iCloseY)? $iOpenY:$iCloseY;
                my $iIeMin = ($iOpenY>$iCloseY)? $iOpenY:$iCloseY;
                $self->{graph}->line( $m, $iHighY, $m, $iIeMax, $dsci)if($iIeMax > $iHighY);
                $self->{graph}->line( $m, $iLowY,  $m, $iIeMin, $dsci)if($iIeMin < $iLowY);
            }
            else {
        #Draw Barcart(Open-High-Low-Close)
                $self->{graph}->line( $l, $iOpenY, $m, $iOpenY,$dsci);
                $self->{graph}->line( $m, $iLowY,  $m, $iHighY, $dsci);
                $self->{graph}->line( $m, $iCloseY,  $r, $iCloseY, $dsci);
            }
        }
    }
    return $ds;
}
"Just another true value";
